package edu.mit.ll.group43.surfaceoptimization;

import java.util.ArrayList;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;

/**
 * Sequencer that schedules aircraft to be sent from the spot
 * immediately when they are ready to do so, eliminating any
 * delay at the spot and concentrating it all at the runway.
 * No explicit optimization is done, the only improvement over
 * real data is that aircraft take off as soon as they can
 * with respect to minimum separation times.
 * 
 * @author William Hawkins
 *
 */
public class FcfsSequencer extends Sequencer {

	public FcfsSequencer(int cps,
			MinimumSeparationCalculator<? extends WeightClass> calc) {
		super(cps, calc);
	}

	/**
	 * Simply sets the optimal release times of the aircraft in
	 * the given sequence to their respective spot ready times
	 * and return the sequence. No re-sequencing is done.
	 * 
	 */
	@Override
	public ArrayList<Aircraft> sequence(ArrayList<Aircraft> seq) {
		for (Aircraft ac : seq) {
			ac.setOptReleaseTime(ac.getSpotReadyTime());
		}
		return seq;
	}

}
