package edu.mit.ll.group43.surfaceoptimization.dp.factory;

import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.cost.CostStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.cost.RunwayCostStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.edge.EdgeValueStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.edge.MinSeparationEdgeValueStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.netgen.CPSNetworkGenerator;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.netgen.GeneratorNetworkCPSNetworkGenerator;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.node.CPSNetworkNodeFactory;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.node.DefaultCPSNetworkNodeFactory;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.sequencerecovery.BacktrackSequenceRecoveryStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.sequencerecovery.SequenceRecoveryStrategy;

/**
 * Factory implementation for a dynamic programming sequencer that
 * maximizes throughput at the runway.
 * 
 * @author William Hawkins
 *
 */
public class RunwayDPSequencerFactory extends DPSequencerFactory {

	public RunwayDPSequencerFactory(int cps,
			MinimumSeparationCalculator<? extends WeightClass> calc) {
		super(cps, calc);
	}

	@Override
	public CostStrategy getCostStrategy() {
		return new RunwayCostStrategy(cps, calc);
	}

	@Override
	public EdgeValueStrategy getEdgeValueStrategy() {
		return new MinSeparationEdgeValueStrategy(calc);
	}
	
	@Override
	public CPSNetworkNodeFactory getCPSNetworkNodeFactory() {
		return new DefaultCPSNetworkNodeFactory();
	}
	
	@Override
	public SequenceRecoveryStrategy getSequenceRecoveryStrategy() {
		return new BacktrackSequenceRecoveryStrategy();
	}
	
	@Override
	public CPSNetworkGenerator getCPSNetworkGenerator() {
		return new GeneratorNetworkCPSNetworkGenerator(cps,
				getCPSNetworkNodeFactory(), getEdgeValueStrategy());
	}
}
