package edu.mit.ll.group43.surfaceoptimization.dp.factory.edge;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;
import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator.ActionType;
import edu.mit.ll.group43.surfaceoptimization.network.NetworkEdge;
import edu.mit.ll.group43.surfaceoptimization.network.dp.CPSNetworkNode;

/**
 * Edge value strategy that simply sets the minimum separation times
 * as the edge weights, and does no further calculations before
 * returning those weights upon request.
 * 
 * @author William Hawkins
 *
 */
public class MinSeparationEdgeValueStrategy extends EdgeValueStrategy {

	public MinSeparationEdgeValueStrategy(MinimumSeparationCalculator<? extends WeightClass> calc) {
		super(calc);
	}
	
	/**
	 * Inserts an edge between a and b, with a weight equal to the
	 * minimum separation time between a and b.
	 * 
	 */
	@Override
	public void insertEdge(CPSNetworkNode a, CPSNetworkNode b) {
		Aircraft prev = a.getLastAircraft();
		Aircraft next = b.getLastAircraft();
		int separation = CALC.getSeparation(prev.getWeightClass(),
				ActionType.DEPARTURE, next.getWeightClass(), ActionType.DEPARTURE);
		NetworkEdge edge = new NetworkEdge(a, b, separation); 
		a.addNextNode(edge);
		b.addPrevNode(edge);
	}
}
