package edu.mit.ll.group43.surfaceoptimization.dp.factory.netgen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.TreeSet;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.edge.EdgeValueStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.node.CPSNetworkNodeFactory;
import edu.mit.ll.group43.surfaceoptimization.network.dp.CPSNetworkNode;
import edu.mit.ll.group43.surfaceoptimization.network.dp.Subsequence;

public class IterativeCPSNetworkGenerator extends CPSNetworkGenerator {

	public IterativeCPSNetworkGenerator(int cps,
			CPSNetworkNodeFactory cpsNetworkNodeFactory,
			EdgeValueStrategy edgeValueStrategy) {
		super(cps, cpsNetworkNodeFactory, edgeValueStrategy);
	}

	@Override
	public ArrayList<TreeSet<CPSNetworkNode>> generateCPSNetworkNodes(
			ArrayList<Aircraft> fcfs) {
		
		ArrayList<TreeSet<CPSNetworkNode>> network = new ArrayList<TreeSet<CPSNetworkNode>>();
		HashMap<Integer, ArrayList<Integer>> validLocs = getValidLocs(fcfs.size());
		
		// Initialize the first stage
		TreeSet<CPSNetworkNode> firstStage = new TreeSet<CPSNetworkNode>();
		for (int i : validLocs.get(1)) {
			Subsequence<Aircraft> seq = new Subsequence<Aircraft>();
			seq.add(fcfs.get(i-1).clone());
			firstStage.add(cpsNetworkNodeFactory.getCPSNetworkNode(seq));
		}
		network.add(firstStage);
		
		// Build the rest of the stages
		for (int i = 1; i < fcfs.size(); i ++) {
			TreeSet<CPSNetworkNode> stage = new TreeSet<CPSNetworkNode>();
			TreeSet<Subsequence<Aircraft>> suffixSet = new TreeSet<Subsequence<Aircraft>>();
			for (CPSNetworkNode node : network.get(i-1)) {
				int begin = Math.max(0, node.getAircraft().size() - 2 * CPS);
				int end = node.getAircraft().size();
				suffixSet.add(new Subsequence<Aircraft>(node.getAircraft().subList(begin, end)));
			}
			
			for (int ac : validLocs.get(i+1)) {
				Aircraft last = fcfs.get(ac-1);
				for (Subsequence<Aircraft> suff : suffixSet) {
					if (!suff.contains(last)) {
						Subsequence<Aircraft> seq = new Subsequence<Aircraft>(suff);
						seq.add(last.clone());
						stage.add(cpsNetworkNodeFactory.getCPSNetworkNode(seq));
					}
				}
			}
			network.add(stage);
		}
		
		return network;
	}

}
