package edu.mit.ll.group43.surfaceoptimization.dp.factory.sequencerecovery;

import java.util.ArrayList;
import java.util.Stack;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.network.NetworkEdge;
import edu.mit.ll.group43.surfaceoptimization.network.dp.CPSNetworkNode;

public class BacktrackSequenceRecoveryStrategy implements SequenceRecoveryStrategy {

	@Override
	public ArrayList<Aircraft> recoverSequence(CPSNetworkNode optEnd) {
		
		// Recursively retrieve its optimal previous nodes
		Stack<CPSNetworkNode> path = new Stack<CPSNetworkNode>();
		path.push(optEnd);
		NetworkEdge optPrev = optEnd.getOptPrevNode();
		while (optPrev != null) {
			CPSNetworkNode optPrevNode = (CPSNetworkNode) optPrev.getSource();
			path.push(optPrevNode);
			optPrev = optPrevNode.getOptPrevNode();
		}
		
		// Build the sequence of aircraft from the optimal path
		ArrayList<Aircraft> optSequence = new ArrayList<Aircraft>();
		while (path.size() > 0) {
			optSequence.add(path.pop().getLastAircraft());
		}
		
		return optSequence;
	}
	
	@Override
	public void processPrevNodes(CPSNetworkNode node) {
		node.setPrevNodes(null);
	}
}
