package edu.mit.ll.group43.surfaceoptimization.network.dp;

import java.util.ArrayList;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;

/**
 * A variant of a CPS network node that keeps track of the optimal
 * release sequence that is unique to its optimal release time.
 * Useful for algorithms that "force" aircraft into gaps they do
 * not fit in without increasing the delay of the aircraft they
 * are trying to fit before in the projected takeoff sequence.
 * 
 * @author William Hawkins
 *
 */
public class SmartCPSNetworkNode extends CPSNetworkNode {

	ArrayList<Aircraft> releaseSequence;
	
	public SmartCPSNetworkNode(Subsequence<Aircraft> aircraft) {
		super(aircraft);
		releaseSequence = new ArrayList<Aircraft>();
	}

	/**
	 * Returns this node's stored spot release sequence.
	 * 
	 * @return
	 */
	public ArrayList<Aircraft> getReleaseSequence() {
		return releaseSequence;
	}

	/**
	 * Sets this node's spot release sequence.
	 * 
	 * @param releaseSequence
	 */
	public void setReleaseSequence(ArrayList<Aircraft> releaseSequence) {
		this.releaseSequence = releaseSequence;
	}
}
