package edu.mit.ll.group43.surfaceoptimization.validator;

import java.util.ArrayList;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;

/**
 * Validator for the constraint that aircraft may not be released
 * from the spot until they are ready at the spot.
 * 
 * @author William Hawkins
 *
 */
public class ReleasedWhenReadyValidator implements Validator {

	/**
	 * Validates that aircraft are not released from the spot until
	 * they are ready at the spot.
	 * 
	 * @return True if valid, false otherwise
	 */
	@Override
	public boolean validate(ArrayList<Aircraft> seq) {
		for (Aircraft ac : seq) {
			if (ac.getOptReleaseTime() < ac.getSpotReadyTime()) {
				return false;
			}
		}
		return true;
	}

}
