package edu.mit.ll.group43.surfaceoptimization.dp.factory.cost;

import java.util.ArrayList;
import java.util.Collections;

import junit.framework.TestCase;

import org.junit.Test;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.AircraftSpotReleaseComparator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.DefaultWeightClass;

public class CostStrategyUtilsTest extends TestCase {

	private ArrayList<Aircraft> createReleaseSequence(int[] indexes, int[] spotReadyTimes,
			int[] optReleaseTimes) {
		ArrayList<Aircraft> releaseSequence = new ArrayList<Aircraft>();
		for (int i = 0; i < indexes.length; i ++) {
			Aircraft ac = new Aircraft(DefaultWeightClass.LARGE, indexes[i], spotReadyTimes[i], 100);
			ac.setFcfsIndex(indexes[i]);
			ac.setOptReleaseTime(optReleaseTimes[i]);
			releaseSequence.add(ac);
		}
		Collections.sort(releaseSequence, AircraftSpotReleaseComparator.getInstance());
		return releaseSequence;
	}
	
	private void testSoiCalculation(ArrayList<Aircraft> sequence, int fcfs, int srt, int cps, int expSoi) {
		Aircraft current = new Aircraft(DefaultWeightClass.LARGE, fcfs, srt, 100);
		current.setFcfsIndex(fcfs);
		int soi = CostStrategyUtils.safePrevTimeBound(sequence, current, cps);
		assertEquals(expSoi, soi);
	}
	
	@Test
	public void testEarliestAchieved() {
		int[] indexes = {1, 2};
		int[] spotReadyTimes = {0, 10};
		int[] optReleaseTimes = {0, 10};
		ArrayList<Aircraft> sequence = createReleaseSequence(indexes, spotReadyTimes, optReleaseTimes);
		
		testSoiCalculation(sequence, 3, 10, 1, 0);
	}
	
	@Test
	public void testCurrentFcfsNotViolated() {
		int[] indexes = {1, 2, 3, 4, 5};
		int[] spotReadyTimes = {0, 10, 20, 30, 40};
		int[] optReleaseTimes = {0, 10, 20, 30, 40};
		ArrayList<Aircraft> sequence = createReleaseSequence(indexes, spotReadyTimes, optReleaseTimes);
		
		// 1-CPS
		testSoiCalculation(sequence, 6, 50, 1, 30);
		testSoiCalculation(sequence, 7, 60, 1, 40);
		
		// 2-CPS
		testSoiCalculation(sequence, 6, 50, 2, 20);
		testSoiCalculation(sequence, 7, 60, 2, 30);
		
		// Test the situation where the aircraft that must come before the one
		// currently being placed, and the next one in the sequence have the
		// same optimal spot release time
		optReleaseTimes[3] = 40;
		sequence = createReleaseSequence(indexes, spotReadyTimes, optReleaseTimes);
		testSoiCalculation(sequence, 6, 50, 1, 40);
		testSoiCalculation(sequence, 7, 50, 2, 40);
	}
	
	@Test
	public void testSequenceCpsPreserved() {
		int[] indexes = {1, 2, 3, 5, 4};
		int[] spotReadyTimes = {0, 10, 20, 40, 30};
		int[] optReleaseTimes = {0, 10, 20, 40, 50};
		ArrayList<Aircraft> sequence = createReleaseSequence(indexes, spotReadyTimes, optReleaseTimes);
		
		// Test that 4's position is respected under 1-CPS
		testSoiCalculation(sequence, 6, 50, 1, 50);
		testSoiCalculation(sequence, 7, 50, 1, 50);
		
		// Test that 2-CPS allows 4 to be moved a second time
		testSoiCalculation(sequence, 6, 50, 2, 20);
		testSoiCalculation(sequence, 7, 50, 2, 40);
	}
	
	@Test
	public void testSmallSequence() {
		int[] indexes = {1};
		int[] spotReadyTimes = {0};
		int[] optReleaseTimes = {0};
		ArrayList<Aircraft> sequence = createReleaseSequence(indexes, spotReadyTimes, optReleaseTimes);
		
		testSoiCalculation(sequence, 2, 10, 1, 0);
		testSoiCalculation(sequence, 3, 10, 1, 0);
		
		int[] indexes2 = {1, 2};
		int[] spotReadyTimes2 = {0, 5};
		int[] optReleaseTimes2 = {0, 10};
		sequence = createReleaseSequence(indexes2, spotReadyTimes2, optReleaseTimes2);
		
		// A/C 3 is ready to leave at time 10, but can be placed
		// anywhere, so it should get 0 back
		testSoiCalculation(sequence, 3, 10, 2, 0);
		
		// Although the sequence is short, we expect an soi of 10
		// instead of A/C 5's spot ready time of 8 because allowing
		// 5 to go at time 8 would put it in position 2, which would
		// violate 2-CPS
		testSoiCalculation(sequence, 5, 8, 2, 10);
	}
}
